/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.petra.concurrent;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

/**
 * @author Shuyang Zhou
 */
public class BaseFutureListener<T> implements FutureListener<T> {

	@Override
	public final void complete(Future<T> future) {
		if (future.isCancelled()) {
			completeWithCancel(future);

			return;
		}

		try {
			completeWithResult(future, future.get());
		}
		catch (Throwable t) {
			if (t instanceof ExecutionException) {
				t = t.getCause();
			}

			completeWithException(future, t);
		}
	}

	public void completeWithCancel(Future<T> future) {
	}

	public void completeWithException(Future<T> future, Throwable throwable) {
	}

	public void completeWithResult(Future<T> future, T result) {
	}

}