/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.petra.concurrent;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * @author Shuyang Zhou
 */
public class ConcurrentIdentityHashMap<K, V>
	extends ConcurrentMapperHashMap<K, IdentityKey<K>, V, V> {

	public ConcurrentIdentityHashMap() {
		this(new ConcurrentHashMap<IdentityKey<K>, V>());
	}

	public ConcurrentIdentityHashMap(
		ConcurrentMap<IdentityKey<K>, V> innerConcurrentMap) {

		super(innerConcurrentMap);
	}

	public ConcurrentIdentityHashMap(int initialCapacity) {
		this(new ConcurrentHashMap<IdentityKey<K>, V>(initialCapacity));
	}

	public ConcurrentIdentityHashMap(
		int initialCapacity, float loadFactor, int concurrencyLevel) {

		this(
			new ConcurrentHashMap<IdentityKey<K>, V>(
				initialCapacity, loadFactor, concurrencyLevel));
	}

	public ConcurrentIdentityHashMap(Map<? extends K, ? extends V> map) {
		this(new ConcurrentHashMap<IdentityKey<K>, V>());

		putAll(map);
	}

	@Override
	protected IdentityKey<K> mapKey(K key) {
		return new IdentityKey<>(key);
	}

	@Override
	protected IdentityKey<K> mapKeyForQuery(K key) {
		return new IdentityKey<>(key);
	}

	@Override
	protected V mapValue(K key, V value) {
		return value;
	}

	@Override
	protected V mapValueForQuery(V value) {
		return value;
	}

	@Override
	protected K unmapKey(IdentityKey<K> identityKey) {
		return identityKey.getKey();
	}

	@Override
	protected K unmapKeyForQuery(IdentityKey<K> identityKey) {
		return identityKey.getKey();
	}

	@Override
	protected V unmapValue(V value) {
		return value;
	}

	@Override
	protected V unmapValueForQuery(V value) {
		return value;
	}

}